@@ -6,6 +6,8 @@ Distributed as part of f90nml, Copyright 2014 Marshall Ward
 Licensed under the Apache License, Version 2.0
 http://www.apache.org/licenses/LICENSE-2.0
 """
+
+import itertools
 import os
 import shlex
 
@@ -13,10 +15,13 @@ __version__ = '0.2'
 
 #---
 def read(nml_fname):
+    """Parse a Fortran 90 namelist file and store the contents in a ``dict``.
 
-    f = open(nml_fname, 'r')
+    >>> data_nml = f90nml.read('data.nml')"""
 
-    f90 = shlex.shlex(f)
+    nml_file = open(nml_fname, 'r')
+
+    f90 = shlex.shlex(nml_file)
     f90.commenters = '!'
     f90.escapedquotes = '\'"'
     f90.wordchars += '.-+'      # Include floating point characters
@@ -49,13 +54,11 @@ def read(nml_fname):
 
             # Skip commas
             if t == ',':
-                prior_t = t
-                t = next(tokens)
+                t, prior_t = next(tokens), t
 
             # Advance token
             if not t == '/':
-                prior_t = t
-                t = next(tokens)
+                t, prior_t = next(tokens), t
 
             if v_name:
 
@@ -63,17 +66,15 @@ def read(nml_fname):
                 # Parse the prior token value
 
                 # Skip variable names and end commas
-                #if not t in ('(', '=') and not (prior_t, t) == (',', '/'):
-                if not ( t == '=' or (t == '(' and not prior_t == '=')
+                if not (t == '=' or (t == '(' and not prior_t == '=')
                         or (prior_t, t) == (',', '/')  or prior_t == ')'):
 
                     # Skip ahead on first value
                     if prior_t == '=':
-                        prior_t = t
-                        t = next(tokens)
+                        t, prior_t = next(tokens), t
 
                     # Parse the variable string
-                    if prior_t == ',' :
+                    if prior_t == ',':
                         next_value = None
                     else:
                         next_value, t = parse_f90val(tokens, t, prior_t)
@@ -111,32 +112,36 @@ def read(nml_fname):
             # Parse the indices of the current variable
             if t == '(' and not prior_t == '=':
                 v_name, v_indices, t = parse_f90idx(tokens, t, prior_t)
-                v_idx = gen_index(v_indices)
 
-            if t == '=':
-                # Activate the next variable
-                if not v_name:
-                    v_name = prior_t
-                    v_idx = None
+                # TODO: Multidimensional support
+                # TODO: End index support (currently ignored)
+                i_s = 1 if not v_indices[0][0] else v_indices[0][0]
+                i_r = 1 if not v_indices[0][2] else v_indices[0][2]
+                v_idx = itertools.count(i_s, i_r)
+
+            if t == '=' and not v_name:
+                v_name = prior_t
+                v_idx = None
 
         # Append the grouplist to the namelist (including empty groups)
         nmls[g_name] = g_vars
 
-    f.close()
+    nml_file.close()
 
     return nmls
 
 
 #---
 def write(nml, nml_fname):
+    """Output dict to a Fortran 90 namelist file."""
 
     if os.path.isfile(nml_fname):
         raise IOError('File {} already exists.'.format(nml_fname))
 
-    f = open(nml_fname, 'w')
+    nml_file = open(nml_fname, 'w')
 
     for grp in sorted(nml.keys()):
-        f.write('&{}\n'.format(grp))
+        nml_file.write('&{}\n'.format(grp))
 
         grp_vars = nml[grp]
         for v_name in sorted(grp_vars.keys()):
@@ -148,34 +153,32 @@ def write(nml, nml_fname):
             else:
                 v_str = to_f90str(v_val)
 
-            f.write('    {} = {}\n'.format(v_name, v_str))
+            nml_file.write('    {} = {}\n'.format(v_name, v_str))
 
-        f.write('/\n')
+        nml_file.write('/\n')
 
-    f.close()
+    nml_file.close()
 
 
 #---
-def to_f90str(v):
+def to_f90str(value):
     """Convert primitive Python types to equivalent Fortran strings"""
 
-    # TODO: Hash this somehow
-    if type(v) is int:
-        return str(v)
-    elif type(v) is float:
-        # TODO: Floating point precision?
-        return str(v)
-    elif type(v) is bool:
-        return '.{}.'.format(str(v).lower())
-    elif type(v) is complex:
-        return '({}, {})'.format(v.real, v.imag)
-    elif type(v) is str:
-        return '\'{}\''.format(v)
-    elif v is None:
+    if type(value) is int:
+        return str(value)
+    elif type(value) is float:
+        return str(value)
+    elif type(value) is bool:
+        return '.{}.'.format(str(value).lower())
+    elif type(value) is complex:
+        return '({}, {})'.format(value.real, value.imag)
+    elif type(value) is str:
+        return '\'{}\''.format(value)
+    elif value is None:
         return ''
     else:
         raise ValueError('Type {} of {} cannot be converted to a Fortran type.'
-                         ''.format(type(v), v))
+                         ''.format(type(value), value))
 
 
 #---
@@ -194,12 +197,12 @@ def parse_f90val(tokens, t, s):
         s = '({}, {})'.format(s_re, s_im)
 
 
-    recast_funcs = [int, float, f90complex, f90bool, f90str]
+    recast_funcs = [int, f90float, f90complex, f90bool, f90str]
 
     for f90type in recast_funcs:
         try:
-            v = f90type(s)
-            return v, t
+            value = f90type(s)
+            return value, t
         except ValueError:
             continue
 
@@ -207,21 +210,31 @@ def parse_f90val(tokens, t, s):
     raise ValueError('Could not convert {} to a Python data type.'.format(s))
 
 
+#---
+def f90float(s):
+    """Convert string repr of Fortran floating point to Python double"""
+
+    # TODO: Distinguish between single and double precision
+    return float(s.lower().replace('d', 'e'))
+
+
 #---
 def f90complex(s):
+    """Convert string repr of Fortran complex to Python complex."""
     assert type(s) == str
 
     if s[0] == '(' and s[-1] == ')' and len(s.split(',')) == 2:
         s_re, s_im = s[1:-1].split(',', 1)
 
         # NOTE: Failed float(str) will raise ValueError
-        return complex(float(s_re), float(s_im))
+        return complex(f90float(s_re), f90float(s_im))
     else:
         raise ValueError('{} must be in complex number form (x, y)'.format(s))
 
 
 #---
 def f90bool(s):
+    """Convert string repr of Fortran logical to Python logical."""
     assert type(s) == str
 
     try:
@@ -239,6 +252,7 @@ def f90bool(s):
 
 #---
 def f90str(s):
+    """Convert string repr of Fortran string to Python string."""
     assert type(s) == str
 
     f90quotes = ["'", '"']
@@ -251,95 +265,76 @@ def f90str(s):
 
 #---
 def parse_f90idx(tokens, t, prior_t):
+    """Parse Fortran vector indices into a tuple of Python indices."""
 
-        idx_end = (',', ')')
+    idx_end = (',', ')')
 
-        v_name = prior_t
-        v_indices = []
-        i_start = i_end = i_stride = None
+    v_name = prior_t
+    v_indices = []
+    i_start = i_end = i_stride = None
 
-        # Start index
+    # Start index
+    t = next(tokens)
+    try:
+        i_start = int(t)
+        t = next(tokens)
+    except ValueError:
+        if t in idx_end:
+            raise ValueError('{} index cannot be empty.'
+                             ''.format(v_name))
+        elif not t == ':':
+            raise
+
+    # End index
+    if t == ':':
         t = next(tokens)
         try:
-            i_start = int(t)
+            i_end = int(t)
             t = next(tokens)
         except ValueError:
-            if t in idx_end:
-                raise ValueError('{} index cannot be empty.'
+            if t == ':':
+                raise ValueError('{} end index cannot be implicit '
+                                 'when using stride.'
                                  ''.format(v_name))
-            elif not t == ':':
+            elif not t in idx_end:
+                raise
+    elif t in idx_end:
+        # Replace index with single-index range
+        if i_start:
+            i_end = i_start
+
+    # Stride index
+    if t == ':':
+        t = next(tokens)
+        try:
+            i_stride = int(t)
+        except ValueError:
+            if t == ')':
+                raise ValueError('{} stride index cannot be '
+                                 'implicit.'.format(v_name))
+            else:
                 raise
 
-        # End index
-        if t == ':':
-            t = next(tokens)
-            try:
-                i_end = int(t)
-                t = next(tokens)
-            except ValueError:
-                if t == ':':
-                    raise ValueError('{} end index cannot be implicit '
-                                     'when using stride.'
-                                     ''.format(v_name))
-                elif not t in idx_end:
-                    raise
-        elif t in idx_end:
-            # Replace index with single-index range
-            if i_start:
-                i_end = i_start
+        if i_stride == 0:
+            raise ValueError('{} stride index cannot be zero.'
+                             ''.format(v_name))
 
-        # Stride index
-        if t == ':':
-            t = next(tokens)
-            try:
-                i_stride = int(t)
-            except ValueError:
-                if t == ')':
-                    raise ValueError('{} stride index cannot be '
-                                     'implicit.'.format(v_name))
-                else:
-                    raise
-
-            if i_stride == 0:
-                raise ValueError('{} stride index cannot be zero.'
-                                 ''.format(v_name))
-
-            t = next(tokens)
-
-        if not t in idx_end:
-            raise ValueError('{} index did not terminate '
-                             'correctly.'.format(v_name))
-
-        idx_triplet = (i_start, i_end, i_stride)
-        v_indices.append((idx_triplet))
         t = next(tokens)
 
-        return v_name, v_indices, t
+    if not t in idx_end:
+        raise ValueError('{} index did not terminate '
+                         'correctly.'.format(v_name))
 
+    idx_triplet = (i_start, i_end, i_stride)
+    v_indices.append((idx_triplet))
+    t = next(tokens)
 
-#---
-def gen_index(idx):
-    # TODO: Multidimensional support (numpy)
-    i_s, i_e, d_i = idx[0]
-
-    if not i_s:
-        i_s = 1
-
-    # TODO: infinite i_e?
-    if not i_e:
-        i_e = 10000
-
-    if not d_i:
-        d_i = 1
-
-    i = i_s
-    while i <= i_e:
-        yield i
-        i += d_i
+    return v_name, v_indices, t
 
 
 #---
 class NmlDict(dict):
+    """Case-insensitive Python dict"""
     def __setitem__(self, key, value):
         super(NmlDict, self).__setitem__(key.lower(), value)
 
